@@ -7,6 +7,7 @@ visualizations of the Doppler modulation of a CW signal.
 """
 
 import os
+
 import matplotlib.pyplot as plt
 
 import pyfstat
@@ -16,6 +17,10 @@ import pyfstat
 # plt.rcParams["font.size"] = 18
 # plt.rcParams["text.usetex"] = True
 
+# workaround deprecation warning
+# see https://github.com/matplotlib/matplotlib/issues/21723
+plt.rcParams["axes.grid"] = False
+
 label = "PyFstat_example_spectrogram"
 outdir = os.path.join("PyFstat_example_data", label)
 
@@ -50,18 +55,18 @@ data = pyfstat.BinaryModulatedWriter(
 data.make_data()
 
 print("Loading SFT data and computing normalized power...")
-times, freqs, sft_data = pyfstat.helper_functions.get_sft_array(data.sftfilepath)
+freqs, times, sft_data = pyfstat.helper_functions.get_sft_as_arrays(data.sftfilepath)
+sft_power = sft_data["H1"].real ** 2 + sft_data["H1"].imag ** 2
 normalized_power = (
-    2 * sft_data**2 / (data_parameters["Tsft"] * data_parameters["sqrtSX"] ** 2)
+    2 * sft_power / (data_parameters["Tsft"] * data_parameters["sqrtSX"] ** 2)
 )
 
 plotfile = os.path.join(outdir, label + ".png")
 print(f"Plotting to file: {plotfile}")
 fig, ax = plt.subplots(figsize=(0.8 * 16, 0.8 * 9))
-ax.grid(which="both")
 ax.set(xlabel="Time [days]", ylabel="Frequency [Hz]", ylim=(99.98, 100.02))
 c = ax.pcolormesh(
-    (times - times[0]) / 86400,
+    (times["H1"] - times["H1"][0]) / 86400,
     freqs,
     normalized_power,
     cmap="inferno_r",
