@@ -50,7 +50,9 @@ class GridSearch(BaseSearchClass):
         "Alpha": r"$\alpha$",
         "Delta": r"$\delta$",
         "twoF": r"$\widetilde{2\mathcal{F}}$",
-        "log10BSGL": r"$\log_{10}\mathcal{B}_{\mathrm{SGL}}$",
+        "maxTwoF": r"$\max\widetilde{2\mathcal{F}}$",
+        "log10BSGL": r"$\log_{10}\mathcal{B}_{\mathrm{S/GL}}$",
+        "lnBtSG": r"$\ln\mathcal{B}_{\mathrm{tS/G}}$",
     }
     """Formatted labels used for plot annotations."""
 
@@ -856,6 +858,25 @@ class TransientGridSearch(GridSearch):
     https://arxiv.org/abs/1805.05652
     for a detailed discussion of the GPU implementation.
 
+    NOTE for GPU users (`tCWFstatMapVersion="pycuda"`):
+    The underlying `ComputeFstat` class tries to
+    conveniently deal with GPU context management behind the scenes.
+    A known problematic case is if you try to instantiate it twice from the same
+    session/script. If you then get some messages like
+    `RuntimeError: make_default_context()`
+    and `invalid device context`,
+    that is because the GPU is still blocked from the first instance when
+    you try to initiate the second.
+    To avoid this problem, use context management::
+
+        with pyfstat.TransientGridSearch(
+            [...],
+            tCWFstatMapVersion="pycuda",
+        ) as search:
+            search.search.run()
+
+    or manually call the `search.search.finalizer_()` method where needed.
+
     Most parameters are the same as for `GridSearch`
     and the `core.ComputeFstat` class,
     only the additional ones are documented here:
@@ -876,6 +897,7 @@ class TransientGridSearch(GridSearch):
         minStartTime=None,
         maxStartTime=None,
         BSGL=False,
+        BtSG=False,
         minCoverFreq=None,
         maxCoverFreq=None,
         detectors=None,
@@ -943,8 +965,12 @@ class TransientGridSearch(GridSearch):
         self.search_keys = ["F0", "F1", "F2", "Alpha", "Delta"]
         for k in self.search_keys:
             setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
-        if self.BSGL:
+        if self.BSGL and self.BtSG:  # pragma: no cover
+            raise ValueError("Please choose only one of [BSGL,BtSG].")
+        elif self.BSGL:
             self.detstat = "log10BSGL"
+        elif self.BtSG:
+            self.detstat = "lnBtSG"
         else:
             self.detstat = "maxTwoF"
         self._initiate_search_object()
@@ -957,6 +983,15 @@ class TransientGridSearch(GridSearch):
                 " results to {}*.dat".format(self.tCWfilebase)
             )
 
+    def __enter__(self):
+        logging.debug("Entering the TransientGridSearch context...")
+        self.search.__enter__()
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        logging.debug("Leaving the TransientGridSearch context...")
+        self.search.__exit__(*args, **kwargs)
+
     def _set_output_keys(self):
         self.output_keys = self.search_keys.copy()
         self.output_keys.append("twoF")
@@ -993,6 +1028,7 @@ class TransientGridSearch(GridSearch):
             minStartTime=self.minStartTime,
             maxStartTime=self.maxStartTime,
             BSGL=self.BSGL,
+            BtSG=self.BtSG,
             SSBprec=self.SSBprec,
             RngMedWindow=self.RngMedWindow,
             injectSources=self.injectSources,
@@ -1140,10 +1176,6 @@ class TransientGridSearch(GridSearch):
         fmt_dict["tau"] = "%d"
         return fmt_dict
 
-    def __del__(self):
-        if hasattr(self, "search"):
-            self.search.__del__()
-
 
 class SliceGridSearch(DefunctClass):
     last_supported_version = "1.9.0"
