@@ -181,12 +181,57 @@ class SignalToNoiseRatio:
 
         return lalpulsar.ComputeOptimalSNR2FromMmunu(Aphys, M)
 
+    def compute_h0_from_snr2(
+        self,
+        Alpha,
+        Delta,
+        psi,
+        phi,
+        cosi,
+        snr2,
+    ):
+        r"""
+        Convert the :math:`\textrm{SNR}^2` of a CW signal to a corresponding amplitude
+        :math:`h_0` given the source orientation.
+        Parameters correspond to the standard ones used to describe a CW
+        (see e.g. Eqs. (16), (26), (30) of https://dcc.ligo.org/T0900149-v6/public ).
+
+        This function returns "inverts" Eq. (77) of
+        https://dcc.ligo.org/T0900149-v6/public by computing the overall prefactor
+        on :math:`h_0` using `self.compute_snr2(h0=1, ...)`.
+
+        Parameters
+        ----------
+        Alpha: float
+            Right ascension (equatorial longitude) of the signal in radians.
+        Delta: float
+            Declination (equatorial latitude) of the signal in radians.
+        psi: float
+            Polarization angle.
+        cosi: float
+            Cosine of the source inclination w.r.t. line of sight.
+            Must be given together with `h0`
+            and conflicts with `aPlus` and `aCross`.
+        snr2: float
+            Squared signal-to-noise ratio of a CW signal
+            in the specified detector network.
+
+        Returns
+        -------
+        h0: float
+            Nominal GW amplitude.
+        """
+        conversion_factor = self.compute_snr2(
+            Alpha=Alpha, Delta=Delta, psi=psi, phi=phi, cosi=cosi, h0=1.0
+        )
+        return np.sqrt(snr2 / conversion_factor)
+
     def compute_twoF(self, *args, **kwargs):
         r"""
         Compute the expected :math:`2\mathcal{F}` value of a CW signal from the result of `compute_snr2`.
 
         .. math:: \langle 2\mathcal{F}\rangle = 4 + \textrm{SNR}^2
-        .. math:: \sigma_{\2\mathcal{F}} =  \sqrt{8 + 4 \textrm{SNR}^2}
+        .. math:: \sigma_{2\mathcal{F}} =  \sqrt{8 + 4 \textrm{SNR}^2}
 
         Input parameters are passed untouched to `self.compute_snr2`.
         See corresponding docstring for a list of valid parameters.
@@ -275,7 +320,7 @@ class DetectorStates:
         """
         Parameters
         ----------
-        time_stamps: array-like or dict
+        timestamps: array-like or dict
             GPS timestamps at which detector states will be retrieved.
             If array, use the same set of timestamps for all detectors,
             which must be explicitly given by the user via `detectors`.
@@ -287,7 +332,7 @@ class DetectorStates:
             separation between consecutive timestamps.
         detectors: list[str] or comma-separated string
             List of detectors to be parsed using XLALParseMultiLALDetector.
-            Conflicts with dictionary of `time_stamps`, required otherwise.
+            Conflicts with dictionary of `timestamps`, required otherwise.
         time_offset: float
             Timestamp offset to retrieve detector states.
             Defaults to LALSuite's default of using the central time of an STF (SFT's timestamp + Tsft/2).
